#include <u.h>
#include <libc.h>
#include <stdio.h>
#include <bio.h>
#include <ndb.h>
#include <ip.h>

void
main(int argc, char **argv)
{
	uchar magic[6 + 16*6];
	uchar *ether;
	uchar *ip;
	uchar *ipmask;
	char addr[255];
	Ndbtuple *tr, *t;
	char *q[] = {"ether", "ip", "ipmask", nil};
	int i, fd;

	if(argc!=2){
		fprint(2, "usage: %s sys\n", argv[0]);
		return;
	}

	fmtinstall('I', eipfmt);

	ip = nil;
	ipmask = nil;
	ether = nil;

	tr = csipinfo(nil, "sys", argv[1], q, 3);
	t = tr;
	while(t){
		if(strcmp(t->attr, "ether")==0){
			ether = malloc(6);
			if(parseether(ether, t->val) < 0){
				fprint(2, "parseether: %r\n");
				return;
			}
		} else
		if(strcmp(t->attr, "ip")==0){
			ip = malloc(IPaddrlen);
			if(parseip(ip, t->val) < 0){
				fprint(2, "parseip: %r\n");
				return;
			}
		} else
		if(strcmp(t->attr, "ipmask")==0){
			ipmask = malloc(IPaddrlen);
			if(parseipmask(ipmask, t->val) < 0){
				fprint(2, "parseipmask: %r\n");
				return;
			}
		}
		t=t->entry;
	}
	ndbfree(tr);

	if(!ether){
		fprint(2, "no ether address found\n");
		return;
	}
	if(!ip){
		fprint(2, "no ip address found\n");
		return;
	}
	if(!ipmask){
		ipmask = malloc(IPaddrlen);
		memcpy(ipmask, defmask(ip), IPaddrlen);
	}

	// make broadcast address
	for(i=0; i<IPaddrlen; i++){
		ip[i] = ip[i] | (~ipmask[i]);
	}

	// make address string
	snprint(addr, sizeof(addr), "udp!%I!2000", ip);

	free(ip);
	free(ipmask);

	// build magic packet
	memset(magic, 0xff, 6);
	for(i=0; i<16; i++){
		memcpy(&magic[i*6+6], ether, 6);
	}

	free(ether);

	fd=dial(addr, 0, 0, 0);
	if(fd < 0){
		fprint(2, "cant dial: %r\n");
		return;
	}
	write(fd, magic, sizeof(magic));
	close(fd);
}
